#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "liste.h"

typedef struct cellule * cellule;

struct cellule  
{ 
  void * info;
  cellule suivant;
};

struct liste
{
  cellule debut;
  cellule fin;
  type_fct_libere libere;
};

liste
lst_cree (type_fct_libere libere)
{
  liste l = (liste) malloc (sizeof (struct cellule));
  assert (l != NULL);

  l->debut = l->fin = NULL;
  l->libere = libere;
  
  return l;
}

void
lst_ajoute_debut (liste l, void * info)
{
  cellule c = (cellule) malloc (sizeof (struct cellule));
  assert (c != NULL);
  
  c->suivant = l->debut;
  c->info = info;
  l->debut = c;
  if (l->fin == NULL)
    l->fin = c;
}

void
lst_ajoute_fin (liste l, void * info)
{
  cellule c = (cellule) malloc (sizeof (struct cellule));
  assert (c != NULL);

  c->suivant = NULL;
  c->info = info;
  if (l->debut == NULL)
    l->debut = c;
  if (l->fin != NULL)
    l->fin->suivant = c;
  l->fin = c;
}

int 
lst_vide (liste l)
{
  return l->debut == NULL;
}

void
lst_applique (liste l, type_fct fct, void * arg)
{ 
  cellule curs = l->debut;
  while (curs != NULL)
    {
      fct (curs->info, arg);
      curs = curs->suivant;
    }
}
  
void
lst_libere (liste l)
{
  cellule curs = l->debut;
  cellule tmp;

  while (curs != NULL)
    {
      tmp = curs;
      curs = curs->suivant;
      l->libere (tmp->info);
      free (tmp);
    }

  free (l);
}

  
